/* ====================================================================
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is SATC.
 *
 * The Initial Developer of the Original Code is Bogdan Mitu
 *
 * Copyright (C) 2001-2002 Bogdan Mitu
 *
 * ====================================================================
 */

header {
package satc.lang;
}

class SugarLexer extends Lexer;

options
{
    k = 3;   
    charVocabulary = '\3'..'\377';
    filter = true; 
    testLiterals = false;
    importVocab = SugarParser;
    exportVocab = Sugar;
}

{
public SugarLexer() {
    this((LexerSharedInputState) null); 
}
}

// Whitespace -- ignored
WS  :   (   ' '
        |   '\t'
        |   '\f'
        // handle newlines
        | ( "\r\n" | '\r' | '\n' )
          { newline(); }
        )
        //{ _ttype = Token.SKIP; }
    ;

// Single-line comments
SL_COMMENT
    :   "--"
        (~('\n'|'\r'))* ('\n'|'\r'('\n')?)
        {
          //$setType(Token.SKIP); 
          newline();
        }
    ;

MLC_OPEN
        : "/*"
        ;


QUESTION        :   '?'     ;
LPAREN          :   '('     ;
RPAREN          :   ')'     ;
LBRACK          :   '['     ;
RBRACK          :   ']'     ;
LCURLY          :   '{'     ;
RCURLY          :   '}'     ;
COLON           :   ':'     ;
COMMA           :   ','     ;
EQ              :   "="     ;
LNOT            :   '!'     ;
NE              :   "!="    ;
DIV             :   '/'     ;
PLUS            :   '+'     ;
MINUS           :   '-'     ;
STAR            :   '*'     ;
MOD             :   '%'     ;
SR              :   ">>"    ;
GE              :   ">="    ;
GT              :   ">"     ;
SL              :   "<<"    ;
LE              :   "<="    ;
LT              :   '<'     ;
BXOR            :   '^'     ;
BOR             :   '|'     ;
LOR             :   "||"    ;
BAND            :   '&'     ;
LAND            :   "&&"    ;
SEMI            :   ';'     ;
TILDE           :   '~'     ;
RANGE           :   ".."    ;
CONC            :   "++"    ;
IFF             :   "<->"   ;
IMPL            :   "->"    ;
IMPL_AX         :   "=>"    ;
SERE_IMPL       :   "|->"   ;
SERE_IMPL_NXT   :   "|=>"   ;
AT              :   "@"     ;



ID  options { testLiterals=true; }
    : (DEC_DIGIT (HEX_DIGIT)* ('h'|'H'))    => (DEC_DIGIT (HEX_DIGIT)* ('h'|'H')) { $setType( NUM_H_INT ); }
    | ((BIN_DIGIT)+ ('b'|'B'))          => ((BIN_DIGIT)+ ('b'|'B'))         { $setType( NUM_B_INT ); }
    | ((DEC_DIGIT)+)                    => (DEC_DIGIT)+                     { $setType( NUM_INT ); }
    
    | ("AG" NOT_ID_CHAR)                => "AG"             { $setType( AG );           }
    | ("AF" NOT_ID_CHAR)                => "AF"             { $setType( AF );           }
    | ("AX" NOT_ID_CHAR)                => "AX"             { $setType( AX );           }
    
    | ("goto" NOT_ID_CHAR)              => "goto"           { $setType( GOTO );         }

    | ("until!_")                       => "until!_"        { $setType( UNTIL_S_I );    }
    | ("until!")                        => "until!"         { $setType( UNTIL_S );      }
    | ("until_" NOT_ID_CHAR)            => "until_"         { $setType( UNTIL_W_I );    }
    | ("until" NOT_ID_CHAR)             => "until"          { $setType( UNTIL_W );      }
    
    | ("before!_")                      => "before!_"       { $setType( BEFORE_S_I );   }
    | ("before!")                       => "before!"        { $setType( BEFORE_S );     }
    | ("before_" NOT_ID_CHAR)           => "before_"        { $setType( BEFORE_W_I );   }
    | ("before"  NOT_ID_CHAR)           => "before"         { $setType( BEFORE_W );     }

    | ("always")                        => "always"         { $setType( ALWAYS );       }
    | ("never")                         => "never"          { $setType( NEVER );        }
    | ("eventually")                    => "eventually"     { $setType( EVENTUALLY );   }

    | ("within!_")                      => "within!_"       { $setType( WITHIN_S_I );   }
    | ("within!")                       => "within!"        { $setType( WITHIN_S );     }
    | ("within_" NOT_ID_CHAR)           => "within_"        { $setType( WITHIN_W_I );   }
    | ("within" NOT_ID_CHAR)            => "within"         { $setType( WITHIN_W );     }
    
    | ("whilenot!_")                    => "whilenot!_"     { $setType( WHILENOT_S_I ); }
    | ("whilenot!")                     => "whilenot!"      { $setType( WHILENOT_S );   }
    | ("whilenot_" NOT_ID_CHAR)         => "whilenot_"      { $setType( WHILENOT_W_I ); }
    | ("whilenot" NOT_ID_CHAR)          => "whilenot"       { $setType( WHILENOT_W );   }
    
    | ("next_event!")                   => "next_event!"    { $setType( NEXT_S );   }
    | ("next_event" NOT_ID_CHAR)        => "next_event"     { $setType( NEXT_W );   }
    
    | ("rose" NOT_ID_CHAR)              => "rose"           { $setType( ROSE ); }
    | ("fell" NOT_ID_CHAR)              => "fell"           { $setType( FELL ); }
    
    | ("true" NOT_ID_CHAR)              => "true"           { $setType( TRUE ); }
    | ("false" NOT_ID_CHAR)             => "false"          { $setType( FALSE ); }
    | ("forall" NOT_ID_CHAR)            => "forall"         { $setType( FORALL ); }
    | ("boolean" NOT_ID_CHAR)           => "boolean"        { $setType( BOOLEAN ); }

    | ('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'0'..'9'|'_')* 
    ;


protected NOT_ID_CHAR
    :  ~('a'..'z'|'A'..'Z'|'0'..'9'|'_')
    ;
    
protected BIN_DIGIT
    :   ('0'..'1')
    ;
    
protected HEX_DIGIT
    :   ('A'..'F' | 'a'..'f' | '0'..'9')
    ;
    
protected DEC_DIGIT
    :   ('0'..'9')
    ;
