/* ====================================================================
 *
 * This class is an adaptation of antlr.debug.misc.JTreeASTModel.
 * See:
 *
 * ANTLR Translator Generator
 * Project led by Terence Parr at http://www.jGuru.com
 * Software rights: http://www.antlr.org/RIGHTS.html
 *
 * $Id: JTreeASTModel.java,v 1.8 2004/02/05 23:51:08 bogdan Exp $
 */




package satc.swing;


import javax.swing.event.EventListenerList;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import satc.ListToken;
import satc.antlr.DocAST;
import antlr.collections.AST;


public class JTreeASTModel implements TreeModel {


    AST root = null;
//    ArrayList listeners; // lazy instantiation
	protected EventListenerList listenerList = new EventListenerList();


    public JTreeASTModel(AST t) {
        if (t == null) {
            throw new IllegalArgumentException("root is null");
        }
        root = t;
    }  
    
    public Object getChild( Object parent, int index) {
        if (parent == null) {
            return null;
        }
		Object returnValue;
		int i = 0;
        DocAST p = (DocAST)parent;
		DocAST c = (DocAST)p.getFirstChild();
		while (c != null) {
			if (i == index) 
				return c;
			i++;
			c = (DocAST)c.getNextSibling();				
		}
		if (p.getParentPN() != null) {
			if (i == index) 
				return "--- parent ---> " + p.getParentPN().toString();
			i++;
		}
		if (p.getFirstToken() != null) {
			if (i == index) 
				return "--- firstToken ---> " + p.getFirstToken().toString();
			i++;
		}
		if (p.getLastToken() != null) {
			if (i == index) 
				return "--- lastToken ---> " + p.getLastToken().toString();
			i++;
		}
		if (p.isDamaged()) {
			if (i == index) 
				return "--- isDamaged() = true";
			i++;				
		}
		throw new ArrayIndexOutOfBoundsException("node has no such child");
	}
    
    public int getChildCount(Object parent) {
        if ( parent==null ) {
            throw new IllegalArgumentException("root is null");
        }
		if (parent instanceof String) 
			return 0;
			
		DocAST p = (DocAST)parent;
		DocAST c = (DocAST)p.getFirstChild();
        int i = 0;
        while ( c!=null ) {
            c = (DocAST)c.getNextSibling();
            i++;
        }
        if (p.getParentPN() != null) i++;
		if (p.getFirstToken() != null) i++;
		if (p.getLastToken() != null) i++;
		if (p.isDamaged()) i++;
		
		return i;
    }  
    
    public int getIndexOfChild(Object parent, Object child) {
        if ( parent==null || child==null ) {
            throw new IllegalArgumentException("root or child is null");
        }
		int i = 0;
		ListToken t;
		DocAST p = (DocAST)parent;
		DocAST c = (DocAST)p.getFirstChild();
        while ( c!=null && c!=child ) {
            c = (DocAST)c.getNextSibling();
            i++;
        }
        if (c == child) {
            return i;
        }
/*
        if ((c = (DocAST)p.getParent()) != null) {
			i++;
        	if (c == child)
        		return i;
        }
*/
		if ((p.getParentPN()) != null) {
			i++;
			if (((String) child).startsWith("--- parent ---> "))
				return i;
		}
		if ((t = p.getFirstToken()) != null) {
			i++;
			if (((String) child).startsWith("--- firstToken ---> "))
				return i;
		}
		if ((t = p.getLastToken()) != null) {
			i++;
			if (((String) child).startsWith("--- lastToken ---> "))
				return i;
		}
		if (p.isDamaged()) {
			i++;
			if (((String) child).startsWith("--- isDamaged()"))
				return i;
		}
        throw new java.util.NoSuchElementException("node is not a child");
    }  

    public Object getRoot() {
        return root;
    } 

    public boolean isLeaf(Object node) {
        if ( node == null ) {
            throw new IllegalArgumentException("node is null");
        }
		if (node instanceof String)
			return true;
        else {
        	AST t = (AST)node;
        	return t.getFirstChild()==null;
        }
    }
    
	/**
	 * Adds a listener for the TreeModelEvent posted after the tree changes.
	 *
	 * @see     #removeTreeModelListener
	 * @param   l       the listener to add
	 */
	public void addTreeModelListener(TreeModelListener l) {
		listenerList.add(TreeModelListener.class, l);
	}

	/**
	 * Removes a listener previously added with <B>addTreeModelListener()</B>.
	 *
	 * @see     #addTreeModelListener
	 * @param   l       the listener to remove
	 */  
	public void removeTreeModelListener(TreeModelListener l) {
		listenerList.remove(TreeModelListener.class, l);
	}

	/**
	 * Returns an array of all the tree model listeners
	 * registered on this model.
	 *
	 * @return all of this model's <code>TreeModelListener</code>s
	 *         or an empty
	 *         array if no tree model listeners are currently registered
	 *
	 * @see #addTreeModelListener
	 * @see #removeTreeModelListener
	 *
	 * @since 1.4
	 */
	public TreeModelListener[] getTreeModelListeners() {
		return (TreeModelListener[])listenerList.getListeners(
				TreeModelListener.class);
	}
    
    public void valueForPathChanged(TreePath path, Object newValue) {
        System.out.println("\nvalueForPathChanged ... \n");
		fireTreeStructureChanged( path.getLastPathComponent(), path);
    }  
    
 
/* ==================================================================
 * 
 *    Borrowed from javax.swing.tree.DefaultTreeModel
 * 
 * ==================================================================
 */
 
 /*
  * Notifies all listeners that have registered interest for
  * notification on this event type.  The event instance 
  * is lazily created using the parameters passed into 
  * the fire method.
  *
  * @param source the node where the tree model has changed
  * @param path the path to the root node
  * @see EventListenerList
  */
  private void fireTreeStructureChanged(Object source, TreePath path) {
	  // Guaranteed to return a non-null array
	  Object[] listeners = listenerList.getListenerList();
	  TreeModelEvent e = null;
	  // Process the listeners last to first, notifying
	  // those that are interested in this event
	  for (int i = listeners.length-2; i>=0; i-=2) {
		  if (listeners[i]==TreeModelListener.class) {
			  // Lazily create the event:
			  if (e == null)
				  e = new TreeModelEvent(source, path);
			  ((TreeModelListener)listeners[i+1]).treeStructureChanged(e);
		  }
	  }
  }

/**
 * @param nodeBefore
 * @return
 */
public TreePath getTreePath( DocAST node) {
	DocAST[] nodes = getTreePathNodes((DocAST) getRoot(), node, 0);
	if (nodes == null)
		return null;
	else
		return new TreePath( nodes);	
}

public DocAST[] getTreePathNodes( DocAST root, DocAST node, int depth) {
	if (node == null) return null;
	depth++;
	DocAST[] retNodes = null;
	if (node == root) {
		retNodes = new DocAST[depth];
		retNodes[depth-1] = root;
	} else {
		int n = root.getNumberOfChildren();
		loop:
		for (int i = 0; i < n; i++) {
			retNodes = getTreePathNodes((DocAST) getChild(root, i), node, depth);
			if (retNodes != null) {
				retNodes[depth-1] = root;
				break loop;
			}
		}
			
	}

	return retNodes;	
}
  
    
}
