header {
package satc.lang; 
}

{
import java.io.*;
import satc.antlr.DocParser;
}

class VerilogParser extends Parser;//(DocParser);

options {
    k = 2;
    importVocab=VerilogLexer;
    exportVocab=Verilog;
    buildAST=true;
}

tokens {
    START;  // imaginary token
	PORT_LIST;
	RANGE;
    UNARY_MINUS;
	UNARY_PLUS;
	SIMPLE_EXPR;
	EXPR;
	NIL;	// NO-MATCH token
	NIL2;	// NO-MATCH token
	NIL3;	// NO-MATCH token
	NIL4;	// NO-MATCH token
	NIL5;	// NO-MATCH token
	NIL6;	// NO-MATCH token
	NIL7;	// NO-MATCH token
}

start   
    : 	(unit | directive)* EOF!  { ## = #([START,"START"], ##) ; }
    ;

unit: 	module
	| 	macromodule
    | 	udp
	;

module 
    : 	"module"^                     		  
      	identifier				       	
       	portList
		SEMI!
      	(element)*
      	"endmodule"!
    ;

portList 
    : 	(LPAREN! identifier (COMMA! identifier)* RPAREN!)?	
		{ ## = #([PORT_LIST,"PORT_LIST"], ##) ; }
    ;

element
	: 	parameterDeclaration 
    | 	inputDeclaration 
    | 	outputDeclaration 
    | 	inoutDeclaration 
    | 	(netDeclaration) => netDeclaration 
    | 	regDeclaration 
//    | 	timeDeclaration 
//    | 	integerDeclaration 
//    | 	realDeclaration 
//    | 	eventDeclaration 
//    | 	gateDeclaration 
//    | 	instantiation 
//    | 	parameterOverride 
    | 	continuousAssign  
//    | 	specifyBlock 
//    | 	initialStatement 
    | 	alwaysStatement 
//    | 	task
//    | 	function
//    | 	directive
    ;

/*------ Declarations ------*/

parameterDeclaration           
    : 	"parameter"^ (range)? parameterAssignmentList SEMI!
    ;

parameterAssignmentList 
	:   parameterAssignment ( COMMA! parameterAssignment )*
    ;

parameterAssignment 
	:   identifier ASSIGN^ expression
    ;

inputDeclaration                               
    : "input"^ (range)? inputList SEMI!
    ;

outputDeclaration 
	:	"output"^ (range)? outputList SEMI!
    ;

inoutDeclaration 
	:	"inout"^ (range)? inoutList SEMI!
    ;

inputList
    : id1:identifier (COMMA! id2:identifier)*
    ;

outputList
    : id1:identifier (COMMA! id2:identifier)*
    ;

inoutList
    : id1:identifier (COMMA! id2:identifier)*
    ;






expression
	:	identifier
	;

range 
	:	LBRACK! simpleExpression COLON! simpleExpression RBRACK! 
		{ ## = #([RANGE,"RANGE"], ##) ; }
    ;

// the mother of all expressions
simpleExpression
	:	shiftExpression
		//{ ## = #([SIMPLE_EXPR,"SIMPLE_EXPR"], ##) ; }
	;

// bit shift expressions (level 4)
shiftExpression
	:	additiveExpression ((SL^ | SR^) additiveExpression)*
	;

// binary addition/subtraction (level 3)
additiveExpression
	:	multiplicativeExpression ((PLUS^ | MINUS^) multiplicativeExpression)*
	;

// multiplication/division/modulo (level 2)
multiplicativeExpression
	:	unaryExpression ((STAR^ | DIV^ | MOD^) unaryExpression)*
	;

unaryExpression
	:	MINUS^ {#MINUS.setType(UNARY_MINUS);} primaryExpression
	|	PLUS^  {#PLUS.setType(UNARY_PLUS);} primaryExpression
	|	primaryExpression
	;

// the basic element of an expression
primaryExpression
	:	identifier
	|	NUMBER
	|	LPAREN simpleExpression RPAREN
	;


identifier 
	:	IDENTIFIER
    |   ESCAPED_IDENTIFIER
    ;

////////////////////////////////////
netDeclaration	:	NIL6	;
regDeclaration	:	NIL7	;

continuousAssign	:	NIL4	;
alwaysStatement	:	NIL5	;

directive: NIL3 ;

macromodule: NIL ;
udp: NIL2 ;